#!/bin/bash

# SUpload (Screenshot-Upload)
version="1.2"

## Colours
r='\033[0;31m'
nc='\033[0m'

## Default  Configs
## WARNING: Do not change these files here, change them in the config file

debug=false
domain="https://volkor.me"
sshot="maim"
photo_format="date +%F-%T.png"
video_format="date +%F-%T.mp4"
ffmpeg_config="-pix_fmt yuv420p -preset ultrafast -vcodec libx264 -tune zerolatency -b:v 3M"
file_dir="${HOME}/Pictures/screenshots"
browser=false
clipboard=true
notification=true
log=true
log_file="${HOME}/.supload.log"
curl_options="--progress-bar"

## Read from user config if exists, else read from system-wide (just use inbuilt defaults if none)
### User config takes priority over system-wide 
config=$HOME/.config/supload/settings.conf 
if [ -f "${config}" ] ; then
  source "${config}"
elif [ -f "/etc/supload.conf" ] ; then
  source "/etc/supload.conf"
fi

## Check dependencies (in case of a bad installation)
type jq >/dev/null 2>&1 || { echo -e >&2 "${r}ERROR:${nc} I require jq but it's not installed.  Aborting."; exit 1; }
type curl >/dev/null 2>&1 || { echo -e >&2 "${r}ERROR:${nc} I require curl but it's not installed.  Aborting."; exit 1; }

## Create screenshot folder if in use.
mkdir -p "$file_dir"

usage() {
cat <<EOF
Usage: supload [OPTIONS] [FILE]

Options:

  -d      Use custom domain use -d <domain>
  -h      Show this menu
  -o      Automatically open in browser
  -s      Screenshot mode (maim Selection)
  -g      Screen Recording Mode (ffcast)
  -V      Verbose (debug mode)
  -v      Shows current version of SUpload

Examples: https://git.volkor.me/Volkor/SUpload

EOF
exit 0
}

## Actual file uploading
upload_file() {
  $debug && echo -e "${r}DEBUG:${nc} BASE_URL: $base_url"

  ## if base_url is still unset (not using -d) set it as the config
  if [ -z ${base_url+x} ]; then
    $debug && echo -e "${r}DEBUG:${nc} BASE_URL unset, replacing with DOMAIN from config file"
    base_url=$domain
    $debug && echo -e "${r}DEBUG:${nc} BASE_URL now set to: $base_url"
  fi

  ## Upload to server (with proper error handling!)
  echo "Uploading File: ($1)"
	response=$(curl $curl_options -F file=@"$1" "$base_url") || { echo -e >&2 "${r}ERROR:${nc} Curl upload failure. (Invalid file, or bad internet connection)"; exit 1; }
  if [[ "${response}" =~ "<html" ]]; then
    $debug && echo -e "${r}DEBUG:${nc} RESONSE: $response"
    echo -e "${r}ERROR:${nc} HTML Curl Error - Website responded in html (file too large? / bad domain?)"
    exit 1
  fi

  $debug && echo -e "${r}DEBUG:${nc} RESPONSE: $response"
  
  url=$(echo "$response" | jq -r '..? | .url // empty') # https://github.com/stedolan/jq/issues/354 allows teknik uploading
  deletionlink=$(echo "$response" | jq -r '..? | .deletionurl // empty') # broken for teknik
  $debug && echo -e "${r}DEBUG:${nc} URL: $url"

  ## Browser, clipboard and logging.
  if [ "$browser" = "true" ] ; then
    echo "Opening browser"
    hash xdg-open && xdg-open "$url" &>/dev/null || echo "${r}ERROR:${nc} xdg-open was not found, or failed to run."
  fi

  if [ "$clipboard" = "true" ] && [ -x "$(command -v xclip)" ]; then
    echo "${url}" | tr -d '\n' | xclip -selection clipboard ; echo "Copied URL to clipboard";
  fi
$debug && echo -e "${r}DEBUG:${nc} BROWSER: $browser"
$debug && echo -e "${r}DEBUG:${nc} CLIPBOARD: $clipboard"

  if [ -f "${log_file}" ]; then
    if [ "$log" = "true" ]; then
      echo -n "$(date +"[%Y-%m-%d %H:%M:%S]")" >> "$log_file" && echo " Upload Response: $response" >> "$log_file"
    fi
  else
    touch "$log_file"
  fi
  $debug && echo -e "${r}DEBUG:${nc} LOG: $log"
  $debug && echo -e "${r}DEBUG:${nc} LOG_FILE: $log_file"
  upload_finished
}

upload_finished() {
  
  if [ -z ${url+x} ]; then
    echo -e "${r}ERROR:${nc} URL was not found in response."
    exit 1
  else
    echo "URL: ${url}"
  fi

  if [ -z ${deletionlink+x} ]; then
    echo -e "${r}DEBUG:${nc} Deletion URL not found :("
  else
    echo "Deletion URL: ${deletionlink}"
  fi
  
  if [ "$notification" = "true" ] ; then
    notify-send "File uploaded." "URL: ${url}" & exit 0
  fi
  exit 0
}

while getopts "d:hosgvV" opt; do
  case $opt in
    d)
      ## TODO: Check if the args are actually set.
	    base_url=${OPTARG}
      ;;
    h)
      usage
      ;;
    o)
	    browser=true
      ;;
    s)
      screenshot=true
      ;;
    g)
      video=true
      ;;
    V)
      debug=true
      ;;
    v)
      echo "SUpload Version: "$version
      exit 0
      ;;
    *)
      echo -e "${r}ERROR:${nc} Invalid Option."
      exit 1
  esac
done

file="${*:$OPTIND}"
$debug && echo -e "${r}DEBUG:${nc} OPTIONS: $*"

# Screenshot
if [ "$screenshot" = "true" ] ; then
  echo "Click and hold to select screenshot area or click for window."
  screen_path="$file_dir/$($photo_format)"
  $debug && echo -e "${r}DEBUG:${nc} SCREEN_PATH: $screen_path"

  case $sshot in
    maim)
      screen_prog="maim -s -o" ;;
    scrot)
      screen_prog="scrot -s" ;;
    *)
      echo -e "${r}ERROR:${nc} Invalid SSHOT in config, must be scrot or maim."
      exit 1
      ;;
  esac
  $debug && echo -e "${r}DEBUG:${nc} SSHOT: $sshot"

  if ! $screen_prog "${screen_path}"; then
    echo -e "${r}ERROR:${nc} Failed to take screenshot ($screen_prog crashed/user exited)"
    exit 1
  fi
  upload_file "${screen_path}"
fi

# Video
if [ "$video" = "true" ] ; then
  video_path="$file_dir/$($video_format)"
  $debug && echo -e "${r}DEBUG:${nc} VIDEO_PATH: $video_path"

  echo -e "${r}WARNING: PRESS ${nc}q${r} to end screen recording!${nc}"
  if !  ffcast -s ffmpeg -- "${ffmpeg_config} ${video_path}"; then
    echo -e "${r}ERROR:${nc} Failed to take screenshot (ffcast crashed/user exited)"
    exit 1
  fi
  upload_file "${video_path}"
fi

if [ "$1" = "" ] ; then
  echo -e "${r}ERROR:${nc} You must supply either STDIN (-) or a file!"
  exit 1
fi

upload_file "$file"